package net.minecraft.client.model;

import net.lax1dude.eaglercraft.v1_8.opengl.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntityHorse;
import net.minecraft.util.MathHelper;

/**
 * + This portion of EaglercraftX contains deobfuscated Minecraft 1.8 source
 * code.
 *
 * Minecraft 1.8.8 bytecode is (c) 2015 Mojang AB. "Do not distribute!" Mod
 * Coder Pack v9.18 deobfuscation configs are (c) Copyright by the MCP Team
 *
 * EaglercraftX 1.8 patch files (c) 2022-2025 lax1dude, ayunami2000. All Rights
 * Reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
public class ModelHorse extends ModelBase {
	private ModelRenderer head;
	private ModelRenderer field_178711_b;
	private ModelRenderer field_178712_c;
	private ModelRenderer horseLeftEar;
	private ModelRenderer horseRightEar;
	private ModelRenderer muleLeftEar;
	private ModelRenderer muleRightEar;
	private ModelRenderer neck;
	private ModelRenderer horseFaceRopes;
	private ModelRenderer mane;
	private ModelRenderer body;
	private ModelRenderer tailBase;
	private ModelRenderer tailMiddle;
	private ModelRenderer tailTip;
	private ModelRenderer backLeftLeg;
	private ModelRenderer backLeftShin;
	private ModelRenderer backLeftHoof;
	private ModelRenderer backRightLeg;
	private ModelRenderer backRightShin;
	private ModelRenderer backRightHoof;
	private ModelRenderer frontLeftLeg;
	private ModelRenderer frontLeftShin;
	private ModelRenderer frontLeftHoof;
	private ModelRenderer frontRightLeg;
	private ModelRenderer frontRightShin;
	private ModelRenderer frontRightHoof;
	private ModelRenderer muleLeftChest;
	private ModelRenderer muleRightChest;
	private ModelRenderer horseSaddleBottom;
	private ModelRenderer horseSaddleFront;
	private ModelRenderer horseSaddleBack;
	private ModelRenderer horseLeftSaddleRope;
	private ModelRenderer horseLeftSaddleMetal;
	private ModelRenderer horseRightSaddleRope;
	private ModelRenderer horseRightSaddleMetal;
	private ModelRenderer horseLeftFaceMetal;
	private ModelRenderer horseRightFaceMetal;
	private ModelRenderer horseLeftRein;
	private ModelRenderer horseRightRein;

	public ModelHorse() {
		this.textureWidth = 128;
		this.textureHeight = 128;
		this.body = new ModelRenderer(this, 0, 34);
		this.body.addBox(-5.0F, -8.0F, -19.0F, 10, 10, 24);
		this.body.setRotationPoint(0.0F, 11.0F, 9.0F);
		this.tailBase = new ModelRenderer(this, 44, 0);
		this.tailBase.addBox(-1.0F, -1.0F, 0.0F, 2, 2, 3);
		this.tailBase.setRotationPoint(0.0F, 3.0F, 14.0F);
		this.setBoxRotation(this.tailBase, -1.134464F, 0.0F, 0.0F);
		this.tailMiddle = new ModelRenderer(this, 38, 7);
		this.tailMiddle.addBox(-1.5F, -2.0F, 3.0F, 3, 4, 7);
		this.tailMiddle.setRotationPoint(0.0F, 3.0F, 14.0F);
		this.setBoxRotation(this.tailMiddle, -1.134464F, 0.0F, 0.0F);
		this.tailTip = new ModelRenderer(this, 24, 3);
		this.tailTip.addBox(-1.5F, -4.5F, 9.0F, 3, 4, 7);
		this.tailTip.setRotationPoint(0.0F, 3.0F, 14.0F);
		this.setBoxRotation(this.tailTip, -1.40215F, 0.0F, 0.0F);
		this.backLeftLeg = new ModelRenderer(this, 78, 29);
		this.backLeftLeg.addBox(-2.5F, -2.0F, -2.5F, 4, 9, 5);
		this.backLeftLeg.setRotationPoint(4.0F, 9.0F, 11.0F);
		this.backLeftShin = new ModelRenderer(this, 78, 43);
		this.backLeftShin.addBox(-2.0F, 0.0F, -1.5F, 3, 5, 3);
		this.backLeftShin.setRotationPoint(4.0F, 16.0F, 11.0F);
		this.backLeftHoof = new ModelRenderer(this, 78, 51);
		this.backLeftHoof.addBox(-2.5F, 5.1F, -2.0F, 4, 3, 4);
		this.backLeftHoof.setRotationPoint(4.0F, 16.0F, 11.0F);
		this.backRightLeg = new ModelRenderer(this, 96, 29);
		this.backRightLeg.addBox(-1.5F, -2.0F, -2.5F, 4, 9, 5);
		this.backRightLeg.setRotationPoint(-4.0F, 9.0F, 11.0F);
		this.backRightShin = new ModelRenderer(this, 96, 43);
		this.backRightShin.addBox(-1.0F, 0.0F, -1.5F, 3, 5, 3);
		this.backRightShin.setRotationPoint(-4.0F, 16.0F, 11.0F);
		this.backRightHoof = new ModelRenderer(this, 96, 51);
		this.backRightHoof.addBox(-1.5F, 5.1F, -2.0F, 4, 3, 4);
		this.backRightHoof.setRotationPoint(-4.0F, 16.0F, 11.0F);
		this.frontLeftLeg = new ModelRenderer(this, 44, 29);
		this.frontLeftLeg.addBox(-1.9F, -1.0F, -2.1F, 3, 8, 4);
		this.frontLeftLeg.setRotationPoint(4.0F, 9.0F, -8.0F);
		this.frontLeftShin = new ModelRenderer(this, 44, 41);
		this.frontLeftShin.addBox(-1.9F, 0.0F, -1.6F, 3, 5, 3);
		this.frontLeftShin.setRotationPoint(4.0F, 16.0F, -8.0F);
		this.frontLeftHoof = new ModelRenderer(this, 44, 51);
		this.frontLeftHoof.addBox(-2.4F, 5.1F, -2.1F, 4, 3, 4);
		this.frontLeftHoof.setRotationPoint(4.0F, 16.0F, -8.0F);
		this.frontRightLeg = new ModelRenderer(this, 60, 29);
		this.frontRightLeg.addBox(-1.1F, -1.0F, -2.1F, 3, 8, 4);
		this.frontRightLeg.setRotationPoint(-4.0F, 9.0F, -8.0F);
		this.frontRightShin = new ModelRenderer(this, 60, 41);
		this.frontRightShin.addBox(-1.1F, 0.0F, -1.6F, 3, 5, 3);
		this.frontRightShin.setRotationPoint(-4.0F, 16.0F, -8.0F);
		this.frontRightHoof = new ModelRenderer(this, 60, 51);
		this.frontRightHoof.addBox(-1.6F, 5.1F, -2.1F, 4, 3, 4);
		this.frontRightHoof.setRotationPoint(-4.0F, 16.0F, -8.0F);
		this.head = new ModelRenderer(this, 0, 0);
		this.head.addBox(-2.5F, -10.0F, -1.5F, 5, 5, 7);
		this.head.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.head, 0.5235988F, 0.0F, 0.0F);
		this.field_178711_b = new ModelRenderer(this, 24, 18);
		this.field_178711_b.addBox(-2.0F, -10.0F, -7.0F, 4, 3, 6);
		this.field_178711_b.setRotationPoint(0.0F, 3.95F, -10.0F);
		this.setBoxRotation(this.field_178711_b, 0.5235988F, 0.0F, 0.0F);
		this.field_178712_c = new ModelRenderer(this, 24, 27);
		this.field_178712_c.addBox(-2.0F, -7.0F, -6.5F, 4, 2, 5);
		this.field_178712_c.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.field_178712_c, 0.5235988F, 0.0F, 0.0F);
		this.head.addChild(this.field_178711_b);
		this.head.addChild(this.field_178712_c);
		this.horseLeftEar = new ModelRenderer(this, 0, 0);
		this.horseLeftEar.addBox(0.45F, -12.0F, 4.0F, 2, 3, 1);
		this.horseLeftEar.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.horseLeftEar, 0.5235988F, 0.0F, 0.0F);
		this.horseRightEar = new ModelRenderer(this, 0, 0);
		this.horseRightEar.addBox(-2.45F, -12.0F, 4.0F, 2, 3, 1);
		this.horseRightEar.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.horseRightEar, 0.5235988F, 0.0F, 0.0F);
		this.muleLeftEar = new ModelRenderer(this, 0, 12);
		this.muleLeftEar.addBox(-2.0F, -16.0F, 4.0F, 2, 7, 1);
		this.muleLeftEar.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.muleLeftEar, 0.5235988F, 0.0F, 0.2617994F);
		this.muleRightEar = new ModelRenderer(this, 0, 12);
		this.muleRightEar.addBox(0.0F, -16.0F, 4.0F, 2, 7, 1);
		this.muleRightEar.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.muleRightEar, 0.5235988F, 0.0F, -0.2617994F);
		this.neck = new ModelRenderer(this, 0, 12);
		this.neck.addBox(-2.05F, -9.8F, -2.0F, 4, 14, 8);
		this.neck.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.neck, 0.5235988F, 0.0F, 0.0F);
		this.muleLeftChest = new ModelRenderer(this, 0, 34);
		this.muleLeftChest.addBox(-3.0F, 0.0F, 0.0F, 8, 8, 3);
		this.muleLeftChest.setRotationPoint(-7.5F, 3.0F, 10.0F);
		this.setBoxRotation(this.muleLeftChest, 0.0F, 1.5707964F, 0.0F);
		this.muleRightChest = new ModelRenderer(this, 0, 47);
		this.muleRightChest.addBox(-3.0F, 0.0F, 0.0F, 8, 8, 3);
		this.muleRightChest.setRotationPoint(4.5F, 3.0F, 10.0F);
		this.setBoxRotation(this.muleRightChest, 0.0F, 1.5707964F, 0.0F);
		this.horseSaddleBottom = new ModelRenderer(this, 80, 0);
		this.horseSaddleBottom.addBox(-5.0F, 0.0F, -3.0F, 10, 1, 8);
		this.horseSaddleBottom.setRotationPoint(0.0F, 2.0F, 2.0F);
		this.horseSaddleFront = new ModelRenderer(this, 106, 9);
		this.horseSaddleFront.addBox(-1.5F, -1.0F, -3.0F, 3, 1, 2);
		this.horseSaddleFront.setRotationPoint(0.0F, 2.0F, 2.0F);
		this.horseSaddleBack = new ModelRenderer(this, 80, 9);
		this.horseSaddleBack.addBox(-4.0F, -1.0F, 3.0F, 8, 1, 2);
		this.horseSaddleBack.setRotationPoint(0.0F, 2.0F, 2.0F);
		this.horseLeftSaddleMetal = new ModelRenderer(this, 74, 0);
		this.horseLeftSaddleMetal.addBox(-0.5F, 6.0F, -1.0F, 1, 2, 2);
		this.horseLeftSaddleMetal.setRotationPoint(5.0F, 3.0F, 2.0F);
		this.horseLeftSaddleRope = new ModelRenderer(this, 70, 0);
		this.horseLeftSaddleRope.addBox(-0.5F, 0.0F, -0.5F, 1, 6, 1);
		this.horseLeftSaddleRope.setRotationPoint(5.0F, 3.0F, 2.0F);
		this.horseRightSaddleMetal = new ModelRenderer(this, 74, 4);
		this.horseRightSaddleMetal.addBox(-0.5F, 6.0F, -1.0F, 1, 2, 2);
		this.horseRightSaddleMetal.setRotationPoint(-5.0F, 3.0F, 2.0F);
		this.horseRightSaddleRope = new ModelRenderer(this, 80, 0);
		this.horseRightSaddleRope.addBox(-0.5F, 0.0F, -0.5F, 1, 6, 1);
		this.horseRightSaddleRope.setRotationPoint(-5.0F, 3.0F, 2.0F);
		this.horseLeftFaceMetal = new ModelRenderer(this, 74, 13);
		this.horseLeftFaceMetal.addBox(1.5F, -8.0F, -4.0F, 1, 2, 2);
		this.horseLeftFaceMetal.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.horseLeftFaceMetal, 0.5235988F, 0.0F, 0.0F);
		this.horseRightFaceMetal = new ModelRenderer(this, 74, 13);
		this.horseRightFaceMetal.addBox(-2.5F, -8.0F, -4.0F, 1, 2, 2);
		this.horseRightFaceMetal.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.horseRightFaceMetal, 0.5235988F, 0.0F, 0.0F);
		this.horseLeftRein = new ModelRenderer(this, 44, 10);
		this.horseLeftRein.addBox(2.6F, -6.0F, -6.0F, 0, 3, 16);
		this.horseLeftRein.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.horseRightRein = new ModelRenderer(this, 44, 5);
		this.horseRightRein.addBox(-2.6F, -6.0F, -6.0F, 0, 3, 16);
		this.horseRightRein.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.mane = new ModelRenderer(this, 58, 0);
		this.mane.addBox(-1.0F, -11.5F, 5.0F, 2, 16, 4);
		this.mane.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.mane, 0.5235988F, 0.0F, 0.0F);
		this.horseFaceRopes = new ModelRenderer(this, 80, 12);
		this.horseFaceRopes.addBox(-2.5F, -10.1F, -7.0F, 5, 5, 12, 0.2F);
		this.horseFaceRopes.setRotationPoint(0.0F, 4.0F, -10.0F);
		this.setBoxRotation(this.horseFaceRopes, 0.5235988F, 0.0F, 0.0F);
	}

	/**
	 * + Sets the models various rotation angles then renders the model.
	 */
	@Override
	public void render(Entity entity, float var2, float var3, float var4, float var5, float var6, float f) {
		EntityHorse entityhorse = (EntityHorse) entity;
		int i = entityhorse.getHorseType();
		float f1 = entityhorse.getGrassEatingAmount(0.0F);
		boolean flag = entityhorse.isAdultHorse();
		boolean flag1 = flag && entityhorse.isHorseSaddled();
		boolean flag2 = flag && entityhorse.isChested();
		boolean flag3 = i == 1 || i == 2;
		float f2 = entityhorse.getHorseSize();
		boolean flag4 = entityhorse.riddenByEntity != null;
		if (flag1) {
			this.horseFaceRopes.render(f);
			this.horseSaddleBottom.render(f);
			this.horseSaddleFront.render(f);
			this.horseSaddleBack.render(f);
			this.horseLeftSaddleRope.render(f);
			this.horseLeftSaddleMetal.render(f);
			this.horseRightSaddleRope.render(f);
			this.horseRightSaddleMetal.render(f);
			this.horseLeftFaceMetal.render(f);
			this.horseRightFaceMetal.render(f);
			if (flag4) {
				this.horseLeftRein.render(f);
				this.horseRightRein.render(f);
			}
		}

		if (!flag) {
			GlStateManager.pushMatrix();
			GlStateManager.scale(f2, 0.5F + f2 * 0.5F, f2);
			GlStateManager.translate(0.0F, 0.95F * (1.0F - f2), 0.0F);
		}

		this.backLeftLeg.render(f);
		this.backLeftShin.render(f);
		this.backLeftHoof.render(f);
		this.backRightLeg.render(f);
		this.backRightShin.render(f);
		this.backRightHoof.render(f);
		this.frontLeftLeg.render(f);
		this.frontLeftShin.render(f);
		this.frontLeftHoof.render(f);
		this.frontRightLeg.render(f);
		this.frontRightShin.render(f);
		this.frontRightHoof.render(f);
		if (!flag) {
			GlStateManager.popMatrix();
			GlStateManager.pushMatrix();
			GlStateManager.scale(f2, f2, f2);
			GlStateManager.translate(0.0F, 1.35F * (1.0F - f2), 0.0F);
		}

		this.body.render(f);
		this.tailBase.render(f);
		this.tailMiddle.render(f);
		this.tailTip.render(f);
		this.neck.render(f);
		this.mane.render(f);
		if (!flag) {
			GlStateManager.popMatrix();
			GlStateManager.pushMatrix();
			float f3 = 0.5F + f2 * f2 * 0.5F;
			GlStateManager.scale(f3, f3, f3);
			if (f1 <= 0.0F) {
				GlStateManager.translate(0.0F, 1.35F * (1.0F - f2), 0.0F);
			} else {
				GlStateManager.translate(0.0F, 0.9F * (1.0F - f2) * f1 + 1.35F * (1.0F - f2) * (1.0F - f1),
						0.15F * (1.0F - f2) * f1);
			}
		}

		if (flag3) {
			this.muleLeftEar.render(f);
			this.muleRightEar.render(f);
		} else {
			this.horseLeftEar.render(f);
			this.horseRightEar.render(f);
		}

		this.head.render(f);
		if (!flag) {
			GlStateManager.popMatrix();
		}

		if (flag2) {
			this.muleLeftChest.render(f);
			this.muleRightChest.render(f);
		}

	}

	/**
	 * + Sets the rotations for a ModelRenderer in the ModelHorse class.
	 */
	private void setBoxRotation(ModelRenderer parModelRenderer, float parFloat1, float parFloat2, float parFloat3) {
		parModelRenderer.rotateAngleX = parFloat1;
		parModelRenderer.rotateAngleY = parFloat2;
		parModelRenderer.rotateAngleZ = parFloat3;
	}

	/**
	 * + Used for easily adding entity-dependent animations. The second and third
	 * float params here are the same second and third as in the setRotationAngles
	 * method.
	 */
	@Override
	public void setLivingAnimations(EntityLivingBase entitylivingbaseIn, float partialTickTime, float parFloat2,
			float parFloat3) {
		super.setLivingAnimations(entitylivingbaseIn, partialTickTime, parFloat2, parFloat3);
		float f = this.updateHorseRotation(entitylivingbaseIn.prevRenderYawOffset, entitylivingbaseIn.renderYawOffset,
				parFloat3);
		float f1 = this.updateHorseRotation(entitylivingbaseIn.prevRotationYawHead, entitylivingbaseIn.rotationYawHead,
				parFloat3);
		float f2 = entitylivingbaseIn.prevRotationPitch
				+ (entitylivingbaseIn.rotationPitch - entitylivingbaseIn.prevRotationPitch) * parFloat3;
		float f3 = f1 - f;
		float f4 = f2 / 57.295776F;
		if (f3 > 20.0F) {
			f3 = 20.0F;
		}

		if (f3 < -20.0F) {
			f3 = -20.0F;
		}

		if (parFloat2 > 0.2F) {
			f4 += MathHelper.cos(partialTickTime * 0.4F) * 0.15F * parFloat2;
		}

		EntityHorse entityhorse = (EntityHorse) entitylivingbaseIn;
		float f5 = entityhorse.getGrassEatingAmount(parFloat3);
		float f6 = entityhorse.getRearingAmount(parFloat3);
		float f7 = 1.0F - f6;
		float f8 = entityhorse.getMouthOpennessAngle(parFloat3);
		boolean flag = entityhorse.field_110278_bp != 0;
		boolean flag1 = entityhorse.isHorseSaddled();
		boolean flag2 = entityhorse.riddenByEntity != null;
		float f9 = (float) entitylivingbaseIn.ticksExisted + parFloat3;
		float f10 = MathHelper.cos(partialTickTime * 0.6662F + 3.1415927F);
		float f11 = f10 * 0.8F * parFloat2;
		this.head.rotationPointY = 4.0F;
		this.head.rotationPointZ = -10.0F;
		this.tailBase.rotationPointY = 3.0F;
		this.tailMiddle.rotationPointZ = 14.0F;
		this.muleRightChest.rotationPointY = 3.0F;
		this.muleRightChest.rotationPointZ = 10.0F;
		this.body.rotateAngleX = 0.0F;
		this.head.rotateAngleX = 0.5235988F + f4;
		this.head.rotateAngleY = f3 / 57.295776F;
		this.head.rotateAngleX = f6 * (0.2617994F + f4) + f5 * 2.18166F
				+ (1.0F - Math.max(f6, f5)) * this.head.rotateAngleX;
		this.head.rotateAngleY = f6 * f3 / 57.295776F + (1.0F - Math.max(f6, f5)) * this.head.rotateAngleY;
		this.head.rotationPointY = f6 * -6.0F + f5 * 11.0F + (1.0F - Math.max(f6, f5)) * this.head.rotationPointY;
		this.head.rotationPointZ = f6 * -1.0F + f5 * -10.0F + (1.0F - Math.max(f6, f5)) * this.head.rotationPointZ;
		this.tailBase.rotationPointY = f6 * 9.0F + f7 * this.tailBase.rotationPointY;
		this.tailMiddle.rotationPointZ = f6 * 18.0F + f7 * this.tailMiddle.rotationPointZ;
		this.muleRightChest.rotationPointY = f6 * 5.5F + f7 * this.muleRightChest.rotationPointY;
		this.muleRightChest.rotationPointZ = f6 * 15.0F + f7 * this.muleRightChest.rotationPointZ;
		this.body.rotateAngleX = f6 * -45.0F / 57.295776F + f7 * this.body.rotateAngleX;
		this.horseLeftEar.rotationPointY = this.head.rotationPointY;
		this.horseRightEar.rotationPointY = this.head.rotationPointY;
		this.muleLeftEar.rotationPointY = this.head.rotationPointY;
		this.muleRightEar.rotationPointY = this.head.rotationPointY;
		this.neck.rotationPointY = this.head.rotationPointY;
		this.field_178711_b.rotationPointY = 0.02F;
		this.field_178712_c.rotationPointY = 0.0F;
		this.mane.rotationPointY = this.head.rotationPointY;
		this.horseLeftEar.rotationPointZ = this.head.rotationPointZ;
		this.horseRightEar.rotationPointZ = this.head.rotationPointZ;
		this.muleLeftEar.rotationPointZ = this.head.rotationPointZ;
		this.muleRightEar.rotationPointZ = this.head.rotationPointZ;
		this.neck.rotationPointZ = this.head.rotationPointZ;
		this.field_178711_b.rotationPointZ = 0.02F - f8 * 1.0F;
		this.field_178712_c.rotationPointZ = 0.0F + f8 * 1.0F;
		this.mane.rotationPointZ = this.head.rotationPointZ;
		this.horseLeftEar.rotateAngleX = this.head.rotateAngleX;
		this.horseRightEar.rotateAngleX = this.head.rotateAngleX;
		this.muleLeftEar.rotateAngleX = this.head.rotateAngleX;
		this.muleRightEar.rotateAngleX = this.head.rotateAngleX;
		this.neck.rotateAngleX = this.head.rotateAngleX;
		this.field_178711_b.rotateAngleX = 0.0F - 0.09424778F * f8;
		this.field_178712_c.rotateAngleX = 0.0F + 0.15707964F * f8;
		this.mane.rotateAngleX = this.head.rotateAngleX;
		this.horseLeftEar.rotateAngleY = this.head.rotateAngleY;
		this.horseRightEar.rotateAngleY = this.head.rotateAngleY;
		this.muleLeftEar.rotateAngleY = this.head.rotateAngleY;
		this.muleRightEar.rotateAngleY = this.head.rotateAngleY;
		this.neck.rotateAngleY = this.head.rotateAngleY;
		this.field_178711_b.rotateAngleY = 0.0F;
		this.field_178712_c.rotateAngleY = 0.0F;
		this.mane.rotateAngleY = this.head.rotateAngleY;
		this.muleLeftChest.rotateAngleX = f11 / 5.0F;
		this.muleRightChest.rotateAngleX = -f11 / 5.0F;
		float f12 = 1.5707964F;
		float f13 = 4.712389F;
		float f14 = -1.0471976F;
		float f15 = 0.2617994F * f6;
		float f16 = MathHelper.cos(f9 * 0.6F + 3.1415927F);
		this.frontLeftLeg.rotationPointY = -2.0F * f6 + 9.0F * f7;
		this.frontLeftLeg.rotationPointZ = -2.0F * f6 + -8.0F * f7;
		this.frontRightLeg.rotationPointY = this.frontLeftLeg.rotationPointY;
		this.frontRightLeg.rotationPointZ = this.frontLeftLeg.rotationPointZ;
		this.backLeftShin.rotationPointY = this.backLeftLeg.rotationPointY
				+ MathHelper.sin(1.5707964F + f15 + f7 * -f10 * 0.5F * parFloat2) * 7.0F;
		this.backLeftShin.rotationPointZ = this.backLeftLeg.rotationPointZ
				+ MathHelper.cos(4.712389F + f15 + f7 * -f10 * 0.5F * parFloat2) * 7.0F;
		this.backRightShin.rotationPointY = this.backRightLeg.rotationPointY
				+ MathHelper.sin(1.5707964F + f15 + f7 * f10 * 0.5F * parFloat2) * 7.0F;
		this.backRightShin.rotationPointZ = this.backRightLeg.rotationPointZ
				+ MathHelper.cos(4.712389F + f15 + f7 * f10 * 0.5F * parFloat2) * 7.0F;
		float f17 = (-1.0471976F + f16) * f6 + f11 * f7;
		float f18 = (-1.0471976F + -f16) * f6 + -f11 * f7;
		this.frontLeftShin.rotationPointY = this.frontLeftLeg.rotationPointY + MathHelper.sin(1.5707964F + f17) * 7.0F;
		this.frontLeftShin.rotationPointZ = this.frontLeftLeg.rotationPointZ + MathHelper.cos(4.712389F + f17) * 7.0F;
		this.frontRightShin.rotationPointY = this.frontRightLeg.rotationPointY
				+ MathHelper.sin(1.5707964F + f18) * 7.0F;
		this.frontRightShin.rotationPointZ = this.frontRightLeg.rotationPointZ + MathHelper.cos(4.712389F + f18) * 7.0F;
		this.backLeftLeg.rotateAngleX = f15 + -f10 * 0.5F * parFloat2 * f7;
		this.backLeftShin.rotateAngleX = -0.08726646F * f6
				+ (-f10 * 0.5F * parFloat2 - Math.max(0.0F, f10 * 0.5F * parFloat2)) * f7;
		this.backLeftHoof.rotateAngleX = this.backLeftShin.rotateAngleX;
		this.backRightLeg.rotateAngleX = f15 + f10 * 0.5F * parFloat2 * f7;
		this.backRightShin.rotateAngleX = -0.08726646F * f6
				+ (f10 * 0.5F * parFloat2 - Math.max(0.0F, -f10 * 0.5F * parFloat2)) * f7;
		this.backRightHoof.rotateAngleX = this.backRightShin.rotateAngleX;
		this.frontLeftLeg.rotateAngleX = f17;
		this.frontLeftShin.rotateAngleX = (this.frontLeftLeg.rotateAngleX
				+ 3.1415927F * Math.max(0.0F, 0.2F + f16 * 0.2F)) * f6
				+ (f11 + Math.max(0.0F, f10 * 0.5F * parFloat2)) * f7;
		this.frontLeftHoof.rotateAngleX = this.frontLeftShin.rotateAngleX;
		this.frontRightLeg.rotateAngleX = f18;
		this.frontRightShin.rotateAngleX = (this.frontRightLeg.rotateAngleX
				+ 3.1415927F * Math.max(0.0F, 0.2F - f16 * 0.2F)) * f6
				+ (-f11 + Math.max(0.0F, -f10 * 0.5F * parFloat2)) * f7;
		this.frontRightHoof.rotateAngleX = this.frontRightShin.rotateAngleX;
		this.backLeftHoof.rotationPointY = this.backLeftShin.rotationPointY;
		this.backLeftHoof.rotationPointZ = this.backLeftShin.rotationPointZ;
		this.backRightHoof.rotationPointY = this.backRightShin.rotationPointY;
		this.backRightHoof.rotationPointZ = this.backRightShin.rotationPointZ;
		this.frontLeftHoof.rotationPointY = this.frontLeftShin.rotationPointY;
		this.frontLeftHoof.rotationPointZ = this.frontLeftShin.rotationPointZ;
		this.frontRightHoof.rotationPointY = this.frontRightShin.rotationPointY;
		this.frontRightHoof.rotationPointZ = this.frontRightShin.rotationPointZ;
		if (flag1) {
			this.horseSaddleBottom.rotationPointY = f6 * 0.5F + f7 * 2.0F;
			this.horseSaddleBottom.rotationPointZ = f6 * 11.0F + f7 * 2.0F;
			this.horseSaddleFront.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.horseSaddleBack.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.horseLeftSaddleRope.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.horseRightSaddleRope.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.horseLeftSaddleMetal.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.horseRightSaddleMetal.rotationPointY = this.horseSaddleBottom.rotationPointY;
			this.muleLeftChest.rotationPointY = this.muleRightChest.rotationPointY;
			this.horseSaddleFront.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.horseSaddleBack.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.horseLeftSaddleRope.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.horseRightSaddleRope.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.horseLeftSaddleMetal.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.horseRightSaddleMetal.rotationPointZ = this.horseSaddleBottom.rotationPointZ;
			this.muleLeftChest.rotationPointZ = this.muleRightChest.rotationPointZ;
			this.horseSaddleBottom.rotateAngleX = this.body.rotateAngleX;
			this.horseSaddleFront.rotateAngleX = this.body.rotateAngleX;
			this.horseSaddleBack.rotateAngleX = this.body.rotateAngleX;
			this.horseLeftRein.rotationPointY = this.head.rotationPointY;
			this.horseRightRein.rotationPointY = this.head.rotationPointY;
			this.horseFaceRopes.rotationPointY = this.head.rotationPointY;
			this.horseLeftFaceMetal.rotationPointY = this.head.rotationPointY;
			this.horseRightFaceMetal.rotationPointY = this.head.rotationPointY;
			this.horseLeftRein.rotationPointZ = this.head.rotationPointZ;
			this.horseRightRein.rotationPointZ = this.head.rotationPointZ;
			this.horseFaceRopes.rotationPointZ = this.head.rotationPointZ;
			this.horseLeftFaceMetal.rotationPointZ = this.head.rotationPointZ;
			this.horseRightFaceMetal.rotationPointZ = this.head.rotationPointZ;
			this.horseLeftRein.rotateAngleX = f4;
			this.horseRightRein.rotateAngleX = f4;
			this.horseFaceRopes.rotateAngleX = this.head.rotateAngleX;
			this.horseLeftFaceMetal.rotateAngleX = this.head.rotateAngleX;
			this.horseRightFaceMetal.rotateAngleX = this.head.rotateAngleX;
			this.horseFaceRopes.rotateAngleY = this.head.rotateAngleY;
			this.horseLeftFaceMetal.rotateAngleY = this.head.rotateAngleY;
			this.horseLeftRein.rotateAngleY = this.head.rotateAngleY;
			this.horseRightFaceMetal.rotateAngleY = this.head.rotateAngleY;
			this.horseRightRein.rotateAngleY = this.head.rotateAngleY;
			if (flag2) {
				this.horseLeftSaddleRope.rotateAngleX = -1.0471976F;
				this.horseLeftSaddleMetal.rotateAngleX = -1.0471976F;
				this.horseRightSaddleRope.rotateAngleX = -1.0471976F;
				this.horseRightSaddleMetal.rotateAngleX = -1.0471976F;
				this.horseLeftSaddleRope.rotateAngleZ = 0.0F;
				this.horseLeftSaddleMetal.rotateAngleZ = 0.0F;
				this.horseRightSaddleRope.rotateAngleZ = 0.0F;
				this.horseRightSaddleMetal.rotateAngleZ = 0.0F;
			} else {
				this.horseLeftSaddleRope.rotateAngleX = f11 / 3.0F;
				this.horseLeftSaddleMetal.rotateAngleX = f11 / 3.0F;
				this.horseRightSaddleRope.rotateAngleX = f11 / 3.0F;
				this.horseRightSaddleMetal.rotateAngleX = f11 / 3.0F;
				this.horseLeftSaddleRope.rotateAngleZ = f11 / 5.0F;
				this.horseLeftSaddleMetal.rotateAngleZ = f11 / 5.0F;
				this.horseRightSaddleRope.rotateAngleZ = -f11 / 5.0F;
				this.horseRightSaddleMetal.rotateAngleZ = -f11 / 5.0F;
			}
		}

		f12 = -1.3089F + parFloat2 * 1.5F;
		if (f12 > 0.0F) {
			f12 = 0.0F;
		}

		if (flag) {
			this.tailBase.rotateAngleY = MathHelper.cos(f9 * 0.7F);
			f12 = 0.0F;
		} else {
			this.tailBase.rotateAngleY = 0.0F;
		}

		this.tailMiddle.rotateAngleY = this.tailBase.rotateAngleY;
		this.tailTip.rotateAngleY = this.tailBase.rotateAngleY;
		this.tailMiddle.rotationPointY = this.tailBase.rotationPointY;
		this.tailTip.rotationPointY = this.tailBase.rotationPointY;
		this.tailMiddle.rotationPointZ = this.tailBase.rotationPointZ;
		this.tailTip.rotationPointZ = this.tailBase.rotationPointZ;
		this.tailBase.rotateAngleX = f12;
		this.tailMiddle.rotateAngleX = f12;
		this.tailTip.rotateAngleX = -0.2618F + f12;
	}

	/**
	 * + Fixes and offsets a rotation in the ModelHorse class.
	 */
	private float updateHorseRotation(float parFloat1, float parFloat2, float parFloat3) {
		float f;
		for (f = parFloat2 - parFloat1; f < -180.0F; f += 360.0F) {
			;
		}

		while (f >= 180.0F) {
			f -= 360.0F;
		}

		return parFloat1 + parFloat3 * f;
	}
}
