/*
 * Copyright (C) 2009 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.common.base;

import java.lang.ref.WeakReference;

import com.google.common.annotations.GwtCompatible;

import net.lax1dude.eaglercraft.v1_8.EagRuntime;

/**
 * Methods factored out so that they can be emulated differently in GWT.
 *
 * @author Jesse Wilson
 */
@GwtCompatible(emulated = true)
final class Platform {
	static <T extends Enum<T>> Optional<T> getEnumIfPresent(Class<T> enumClass, String value) {
		WeakReference<? extends Enum<?>> ref = Enums.getEnumConstants(enumClass).get(value);
		return ref == null ? Optional.<T>absent() : Optional.of(enumClass.cast(ref.get()));
	}

	static CharMatcher precomputeCharMatcher(CharMatcher matcher) {
		return matcher.precomputedInternal();
	}

	/** Calls {@link System#nanoTime()}. */
	static long systemNanoTime() {
		return EagRuntime.nanoTime();
	}

	private Platform() {
	}
}
