/*
 * Copyright (C) 2010 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.common.collect;

import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;

import com.google.common.annotations.Beta;
import com.google.common.annotations.GwtCompatible;

/**
 * Interface that extends {@code Table} and whose rows are sorted.
 *
 * <p>
 * The {@link #rowKeySet} method returns a {@link SortedSet} and the
 * {@link #rowMap} method returns a {@link SortedMap}, instead of the
 * {@link Set} and {@link Map} specified by the {@link Table} interface.
 *
 * @author Warren Dukes
 * @since 8.0
 */
@GwtCompatible
@Beta
public interface RowSortedTable<R, C, V> extends Table<R, C, V> {
	/**
	 * {@inheritDoc}
	 *
	 * <p>
	 * This method returns a {@link SortedSet}, instead of the {@code Set} specified
	 * in the {@link Table} interface.
	 */
	@Override
	SortedSet<R> rowKeySet();

	/**
	 * {@inheritDoc}
	 *
	 * <p>
	 * This method returns a {@link SortedMap}, instead of the {@code Map} specified
	 * in the {@link Table} interface.
	 */
	@Override
	SortedMap<R, Map<C, V>> rowMap();
}
