/*
 * Copyright (c) 2023-2024 lax1dude. All Rights Reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package net.lax1dude.eaglercraft.v1_8.sp.ipc;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

public class IPCPacket15Crashed implements IPCPacketBase {

	public static final int ID = 0x15;

	public String crashReport;

	public IPCPacket15Crashed() {
	}

	public IPCPacket15Crashed(String crashReport) {
		this.crashReport = crashReport;
	}

	@Override
	public void deserialize(DataInput bin) throws IOException {
		int len = bin.readInt();
		byte[] bytes = new byte[len];
		bin.readFully(bytes);
		crashReport = new String(bytes, StandardCharsets.UTF_8);
	}

	@Override
	public int id() {
		return ID;
	}

	@Override
	public void serialize(DataOutput bin) throws IOException {
		byte[] bytes = crashReport.getBytes(StandardCharsets.UTF_8);
		bin.writeInt(bytes.length);
		bin.write(bytes);
	}

	@Override
	public int size() {
		return IPCPacketBase.strLen(crashReport) + 2;
	}

}